//
//      AWSListProducts.cpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

#include "pself3.hpp"
#include "AWSListProducts.hpp"

// Constructor
AWSListProducts::AWSListProducts(Void) 
{
    return;
}

// Destructor
AWSListProducts::~AWSListProducts(Void)
{
	_errmsg.Clear();
	
	SFXArray<AWSProductPtr>::Enumerator enumerator = _productarray.GetFirstEnumerator();
    while (enumerator.HasNext()) 
	{
		AWSProductPtr item(enumerator.GetNext());
		delete item;
    }
    _productarray.Clear();

    return;
}

// get i-th product
AWSProductPtr AWSListProducts::GetProduct(UInt16 index)
{
	if (index < _productarray.GetSize()) {
        return _productarray[index];
    }
    return null;
}

// add a product to the list
SFCError AWSListProducts::AddProduct(AWSProductPtr item)
{
	return (_productarray.Append(item));
}

// get list of products from favorite file
SFCError AWSListProducts::GetFavoriteProducts(SFXPathConstRef filename, SFXAnsiString producttype)
{
	SFCError error;
	UInt16 i, size;

	SFXConfig _config;
 	AWSProductPtr item;

    // Load item list from file
    error = _config.Load(filename);
	if (error != SFERR_NO_ERROR)
	{
		_errmsg = "Cannot read from file";
		return error;
	}

    // get number of items
    size = _config.ReadUInt16(0, 0);

    for (i = 0; i < size; i++) {
		//process each item
		item = AWSProduct::Factory(producttype);
		item->SetAsin(_config.ReadSFXAnsiString(i * 2 + 1, ""));
		item->SetTitle(_config.ReadSFXAnsiString(i * 2 + 2, ""));
		
		error = _productarray.Append(item);
		if (error != SFERR_NO_ERROR) break;
    }

	return error;
}

// update favorite file
Void AWSListProducts::UpdateFavoriteFile(SFXPathConstRef filename)
{
 	SFXConfig _config;
	UInt16 i, size;

	size = (UInt16) _productarray.GetSize();
	if (size<=0)
	{
		SFXFile::Remove(filename);
		return;
	}

    _config.Load(filename);
	
	// update number of favorite items 
    _config.WriteUInt16(0, size);

	// update product list to favorite file 
    for (i = 0; i < size; i++) {
        _config.WriteSFXAnsiString(i * 2 + 1, _productarray[i]->GetAsin());
        _config.WriteSFXAnsiString(i * 2 + 2, _productarray[i]->GetTitle());
    }
    _config.Save();
}

// remove a product from list and delete related files
SFCError AWSListProducts::RemoveProduct(SFXAnsiStringConstRef asin)
{
    SInt16 i;
	SFCError error(SFERR_NO_ERROR);
	SFXPathConst filenameXML(DIR_NAME + asin+ ".xml");
	SFXPathConst filenameJPG(DIR_NAME + asin + ".jpg");

	for (i = 0; i < _productarray.GetSize(); i++) 
	{
		if (_productarray[i]->GetAsin() == asin) 
		{
			SFXFile::Remove(filenameXML);
			SFXFile::Remove(filenameJPG);
			delete _productarray[i];
            _productarray.Remove(i);
            
			--i;
            error = SFERR_NO_ERROR;
			break;
        }
    }
    return error;
}

// remove all products from list and delete all related files
SFCError AWSListProducts::RemoveAllProducts()
{
    SFCError error(SFERR_NO_ERROR);
	SFXPath filenameXML, filenameJPG;

	SFXArray<AWSProductPtr>::Enumerator enumerator = _productarray.GetFirstEnumerator();
	while (enumerator.HasNext()) 
	{
		AWSProductPtr item(enumerator.GetNext());
		filenameXML = SFXPath(DIR_NAME + item->GetAsin() + ".xml");
		filenameJPG = SFXPath(DIR_NAME + item->GetAsin() + ".jpg");
		SFXFile::Remove(filenameXML);
		SFXFile::Remove(filenameJPG);
        delete item;
    }
    _productarray.Clear();
	return error;
}



