//
//      AWSProduct.hpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

/**
***	Project         : Amazon Products Viewing
***	File			: AWSProduct.hpp	
***	Description		: Amazon Web Service Products
**/

#ifndef __AWSPRODUCT_HPP
#define __AWSPRODUCT_HPP

#include <SophiaFramework.hpp>

// Paths to three sizes of Amazon product images for each item in the response.
// The three sizes are small, medium, and large.
SFMTYPEDEFSTRUCT(ImageUrlRec)
struct ImageUrlRec {
	SFXAnsiString	smallImageURL;
	SInt32			smallImageHeight;
	SInt32			smallImageWidth;
	SFXAnsiString	mediumImageURL;
	SInt32			mediumImageHeight;
	SInt32			mediumImageWidth;
	SFXAnsiString	largeImageURL;
	SInt32			largeImageHeight;
	SInt32			largeImageWidth;
};

// Number of offer listings and the lowest price for each of the offer listing condition classes, 
// including New, Used, Collectible, and Refurbished
SFMTYPEDEFSTRUCT(OfferSummaryRec)
struct OfferSummaryRec {
	SInt32			lowestNewPrice;
	SFXAnsiString	currencyCode;
	SFXAnsiString	formattedPrice;
	SInt32			totalNew;
	SInt32			totalUsed;
	SInt32			totalCollectible;
	SInt32			totalRefurbished;
};

// Contents of the OfferSummary response group
SFMTYPEDEFSTRUCT(OfferRec)
struct OfferRec {
	SFXAnsiString	merchantId;
	SFXAnsiString	condition;
	SFXAnsiString	offerListingId;
	SInt32			amount;
	SFXAnsiString	CurrencyCode;
	SFXAnsiString	formattedPrice;
};

// List of offers
SFMTYPEDEFSTRUCT(OffersRec)
struct OffersRec {
	SInt32 totalOffers;
	SInt32 totalOfferspages;
	SFXArray<OfferRecPtr> offers;
};

// Each customer review will contain the rating, summary, date of review, and full review text
SFMTYPEDEFSTRUCT(ReviewRec)
struct ReviewRec {
	SFXAnsiString	asin;
	SInt32			rating;
	SInt32			helpfulVotes;
	SFXAnsiString	customerId;
	SInt32			totalVotes;
	SFXDate			date;
	SFXAnsiString	summary;
	SFXAnsiString	content;
};

// List of customer reviews, an average rating (1 to 5 stars), and the total number of reviews for each item in the response
SFMTYPEDEFSTRUCT(CustomerReviewsRec)
struct CustomerReviewsRec {
	SInt32 averageRating;
	SInt32 totalReviews;
	SInt32 totalReviewPages;
	SFXArray<ReviewRecPtr> reviews;
};

// ASIN and Title of each similar product 
SFMTYPEDEFSTRUCT(SimilarProductRec)
struct SimilarProductRec {
	SFXAnsiString asin;
	SFXAnsiString title;
};

// List of similar products
SFMTYPEDEFSTRUCT(SimilarProductsRec)
struct SimilarProductsRec {
	SFXArray<SimilarProductRecPtr> similarProducts;
};

// All products listed on Amazon's websites have been classified. 
// This classification system is called browse 
SFMTYPEDEFSTRUCT(BrowserNodesRec)
struct BrowserNodesRec {
	SFXAnsiString dummy;
	// Not yet implimentation.
};

// Listmania list IDs and list names for each list returned
SFMTYPEDEFSTRUCT(ListmaniaListsRec)
struct ListmaniaListsRec {
	SFXAnsiString dummy;
	// Not yet implimentation.
};

// A common AWS(Amazon Web Services) product.
// is parent class of specific product class such as Book or DVD

SFMTYPEDEFCLASS(AWSProduct)
class AWSProduct {
SFMSEALCOPY(AWSProduct)

protected:	
	
	SFXAnsiString			_flag;	// control flag used by SetItemAttributes method
	/*
	**common attributes for all products
	*/
	SFXAnsiString			_asin;				// Amazon product ASIN
	SFXAnsiString			_detailPageURL;		// detail page URL
	SInt32					_salesRank;			// Amazon sale rank
	SFXAnsiString			_title;				// product title

	ImageUrlRecPtr			_imageUrl;		
	OfferSummaryRecPtr		_offerSummary;
	OffersRecPtr			_offers;
	CustomerReviewsRecPtr	_customerReviews;
	ReviewRecPtr			_review;
	SimilarProductsRecPtr	_similarProducts;
	SimilarProductRecPtr	_similarProduct;
	BrowserNodesRecPtr		_browserNodes;
	ListmaniaListsRecPtr	_listmaniaLists;

public:
	AWSProduct();
	virtual ~AWSProduct(Void);
	static AWSProductPtr Factory(SFXAnsiString producttype);

	static SFXAnsiString GetUrlForItemLookup(SFXAnsiStringConstRef asin);
	static SFXAnsiString GetUrlForKeyWordSearch(SFXAnsiStringConstRef searchindex, SFXAnsiStringConstRef browsenode, SFXAnsiStringConstRef keyword);

	Void	DoNothing(SFXAnsiString param)				{return;}

	Void	SetFlag(SFXAnsiString param)				{_flag = DecodeHtml(param);}
	Void	SetAsin(SFXAnsiString param)				{_asin = DecodeHtml(param);}
	Void	SetDetailPageURL(SFXAnsiString param)		{_detailPageURL = DecodeHtml(param);}
	Void	SetSalesRank(SFXAnsiString param)			{_salesRank = param.Replace(',', "").AsSInt32();}
	Void	SetTitle(SFXAnsiString param)				{_title = DecodeHtml(param);}
	
	Void	NewImageUrl(SFXAnsiString param)			{_imageUrl = new ImageUrlRec;}
	Void	SetSmallImageURL(SFXAnsiString param)		{_imageUrl->smallImageURL	= DecodeHtml(param);}
	Void	SetSmallImageHeight(SFXAnsiString param)	{_imageUrl->smallImageHeight= param.AsSInt32();}
	Void	SetSmallImageWidth(SFXAnsiString param)		{_imageUrl->smallImageWidth = param.AsSInt32();}
	Void	SetMediumImageURL(SFXAnsiString param)		{_imageUrl->mediumImageURL	= DecodeHtml(param);}
	Void	SetMediumImageHeight(SFXAnsiString param)	{_imageUrl->mediumImageHeight = param.AsSInt32();}
	Void	SetMediumImageWidth(SFXAnsiString param)	{_imageUrl->mediumImageWidth  = param.AsSInt32();}
	Void	SetLargeImageURL(SFXAnsiString param)		{_imageUrl->largeImageURL	= DecodeHtml(param);}
	Void	SetLargeImageHeight(SFXAnsiString param)	{_imageUrl->largeImageHeight = param.AsSInt32();}
	Void	SetLargeImageWidth(SFXAnsiString param)		{_imageUrl->largeImageWidth  = param.AsSInt32();}

	Void	NewCustomerReviews(SFXAnsiString param)		{_customerReviews = new CustomerReviewsRec;}	 
	Void	NewReview(SFXAnsiString param)				{_review = new ReviewRec;}
	Void	SetReview(SFXAnsiString param)				{_customerReviews->reviews.Append(_review);}
	Void	SetCRAverageRating(SFXAnsiString param)		{_customerReviews->averageRating = param.AsSInt32();}
	Void	SetCRTotalReviews(SFXAnsiString param)		{_customerReviews->totalReviews = param.AsSInt32();}
	Void	SetCRtotalReviewPages(SFXAnsiString param)	{_customerReviews->totalReviewPages = param.AsSInt32();}
	Void	SetReviewAsin(SFXAnsiString param)			{_review->asin = DecodeHtml(param);}
	Void	SetReviewRating(SFXAnsiString param)		{_review->rating = param.AsSInt32();}
	Void	SetReviewHelpfulVotes(SFXAnsiString param)	{_review->helpfulVotes = param.AsSInt32();}
	Void	SetReviewCustomerId(SFXAnsiString param)	{_review->customerId = DecodeHtml(param);}
	Void	SetReviewTotalVotes(SFXAnsiString param)	{_review->totalVotes = param.AsSInt32();}
	Void	SetReviewDate(SFXAnsiString param)			{_review->date = StringToDate(param);}
	Void	SetReviewSummary(SFXAnsiString param)		{_review->summary = DecodeHtml(param);}
	Void	SetReviewContent(SFXAnsiString param)		{_review->content = DecodeHtml(param);}
	
	Void	NewSimilarProducts(SFXAnsiString param)		{_similarProducts = new SimilarProductsRec;}
	Void	NewSimilarProduct(SFXAnsiString param)		{_similarProduct = new SimilarProductRec;}
	Void	SetSimilarProduct(SFXAnsiString param)		{_similarProducts->similarProducts.Append(_similarProduct);}
	Void	SetSimilarProductAsin(SFXAnsiString param)	{_similarProduct->asin = DecodeHtml(param);}
	Void	SetSimilarProductTitle(SFXAnsiString param)	{_similarProduct->title = DecodeHtml(param);}

	SFXAnsiStringConstRef		GetAsin(Void)			{ return _asin; }
	SFXAnsiStringConstRef		GetDetailPageURL(Void)	{ return _detailPageURL; }
	SInt32						GetSalesRank(Void)		{ return _salesRank; }
	SFXAnsiStringConstRef		GetTitle(Void)			{ return _title; }

	ImageUrlRecConstPtr			GetImageUrl(Void)		{ return _imageUrl; }
	OfferSummaryRecConstPtr		GetOfferSummary(Void)	{ return _offerSummary; }
	OffersRecConstPtr			GetOffers(Void)			{ return _offers; }
	CustomerReviewsRecConstPtr	GetCustomerReviews(Void){ return _customerReviews; }
	SimilarProductsRecConstPtr	GetSimilarProducts(Void){ return _similarProducts; }
	BrowserNodesRecConstPtr		GetBrowserNodes(Void)	{ return _browserNodes; }
	ListmaniaListsRecConstPtr	GetListmaniaLists(Void) { return _listmaniaLists; }

	virtual Void	ClearData();
	virtual Void	NewItemAttributes(SFXAnsiString param) = 0;
	virtual Void	SetItemAttributes(SFXAnsiString param) = 0;
	
    	
	// display product information
	virtual	SFCError MakeGeneralPane(SFRTabControlPtr tab)=0;
	virtual	SFCError MakeDetailPane(SFRTabControlPtr tab)=0;
	virtual	SFCError MakeReviewPane(SFRTabControlPtr tab);
	virtual	SFCError MakeSimilarPane(SFRTabControlPtr tab);
	virtual	SFCError MakeImagePane(SFRTabControlPtr tab);
	virtual	SFCError DisplayMe(SFXAnsiStringConstRef producttype, bool addedFavorite);

	//helper functions
	SFXAnsiString DecodeHtml(SFXAnsiStringConstRef string);
	SFXDate StringToDate(SFXAnsiStringConstRef string);
};

////AWS Book item attributes
SFMTYPEDEFSTRUCT(BookItemAttributesRec)
struct BookItemAttributesRec {
	SFXArray<SFXAnsiStringPtr> authors;
	SFXAnsiString	binding;
	SInt32			amount;
	SFXAnsiString	currencyCode;
	SFXAnsiString	formattedPrice;
	SInt32			numberOfPages;
	SInt32			packageDimensionsLength;
	SInt32			packageDimensionsWidth;
	SFXDate			publicationDate;
	SFXAnsiString	publisher;
};

//Book class, inherits from AWSProduct class
SFMTYPEDEFCLASS(AWSBook)
class AWSBook : public AWSProduct {
	SFMSEALCOPY(AWSBook)

private:
	//SFXAnsiString			 _flag;	
	BookItemAttributesRecPtr _itemAttributes; //Book item attributes

public:
    AWSBook(Void);
    virtual ~AWSBook(Void);

public:
	virtual Void	NewItemAttributes(SFXAnsiString param)		{_itemAttributes = new BookItemAttributesRec;};
	virtual Void	SetItemAttributes(SFXAnsiString param);

	// display book information
	virtual SFCError MakeGeneralPane(SFRTabControlPtr tab);
	virtual SFCError MakeDetailPane(SFRTabControlPtr tab);
};

//AWS DVD item attributes
SFMTYPEDEFSTRUCT(DvdItemAttributesRec)
struct DvdItemAttributesRec {
	SFXArray<SFXAnsiStringPtr> actors;
	SFXArray<SFXAnsiStringPtr> directors;
	SFXArray<SFXAnsiStringPtr> formats;
	SInt32			amount;
	SFXAnsiString	currencyCode;
	SFXAnsiString	formattedPrice;
	SFXAnsiString	publisher;
	SFXDate			releaseDate;
	SInt32			runningTime;	
};

//DVD class, inherits from AWSProduct class
SFMTYPEDEFCLASS(AWSDvd)
class AWSDvd : public AWSProduct {
	SFMSEALCOPY(AWSDvd)

private:
	DvdItemAttributesRecPtr		_itemAttributes; //DVD item attributes

public:
    AWSDvd(Void);
    virtual ~AWSDvd(Void);

	virtual Void	NewItemAttributes(SFXAnsiString param)		{_itemAttributes = new DvdItemAttributesRec;}
	virtual Void	SetItemAttributes(SFXAnsiString param);

	// display DVD information
	virtual SFCError MakeGeneralPane(SFRTabControlPtr tab);
	virtual SFCError MakeDetailPane(SFRTabControlPtr tab);
};
#endif // __AWSPRODUCT_HPP //
