//
//      SGRNetworkDialogSet.cpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

#include "SGRNetworkDialogSet.hpp"
# include "pself3.hpp"

#define HTTP_ERROR_TITLE "HTTP Connect Error"
#define HTTP_ERROR_CODE1 "Unable to establish connection"
#define HTTP_ERROR_CODE2 "Connecting failed"
#define HTTP_ERROR_CODE3 "Unable to receive data" 
#define DIALOG_OK "OK"

SGRNetworkDialogSet::SGRNetworkDialogSet(Void):	
	_dialog(null),_connecting(false)
{
    return;
}

SGRNetworkDialogSet::~SGRNetworkDialogSet(Void)
{
    return;
}

// Open Dialog
SFCError SGRNetworkDialogSet::Open(SFXAnsiStringConstRef url, Bool isConfirm, NetworkDialogSetSPP procedure, VoidPtr data, SFXAnsiStringConstRef title, SFXAnsiStringConstRef text, SFXAnsiStringConstRef ok, SFXAnsiStringConstRef cancel)
{
    SFCError error(SFERR_NO_ERROR);

	SFXSize devsize;
	devsize = SFXGraphics::GetDeviceSize();

	SInt16Const top = devsize.GetHeight() / 4;
	SInt16Const left = devsize.GetWidth() / 12;
	SInt16Const width = devsize.GetWidth() - left * 2;
	SInt16Const height = devsize.GetHeight() - top * 2;

    _url = url;
    _spp = procedure;
    _reference = data;
	_title = title;
	_text = text;
	_ok = ok;
	_cancel = cancel;
	
    if (isConfirm) { // display connect confirm dialog
        SFRMultiDialog::ParamRec param;
        param.text = _text;
        param.ok = _ok;
        param.cancel = _cancel;
        param.another = "";                      // empty for not displaying
        param.focus = SFRMultiDialog::FOCUS_OK,  // focus OK button
        param.image = SFBImageSmp(null);         // no image
        
		_dialog = ::new SFRMultiDialog( SFXRectangle(left, top ,width, height), _title, param);
		if (_dialog == null) return SFERR_NO_MEMORY;

		error = SGRSoftkeyToolbox::OpenStandardDialog(_dialog, HANDLER_FUNCTION(OnDialog));
    } 
	else 
		error = Connect(); // HTTP connect
    
    return error;
}

// HTTP connect
SFCError SGRNetworkDialogSet::Connect(Void)
{
	SFCError error;
	
	_connecting = true;

	// open connection
	if ((error = _http.Open()) == SFERR_NO_ERROR) {
		if ((error = _http.SetMethod("GET")) == SFERR_NO_ERROR) {
			// Make a call to ECS and get the response by callback function
			error = _http.Connect(_url, CALLBACK_FUNCTION(ConnectCallback));
        }
    }
	
	//notify if error occured
    if (error != SFERR_NO_ERROR) {
		_http.Close();
		SGRSoftkeyToolbox::CloseStandardDialog(_dialog);
		_spp(NOTIFY_CONNECT_START, error, _reference);
		_errorDialog.Open(HTTP_ERROR_TITLE, HTTP_ERROR_CODE1, DIALOG_OK, this);
    }

    return error;
}

// OnDialog handler
HANDLER_IMPLEMENT_VOIDDIALOG(SGRNetworkDialogSet, OnDialog, result, dialog)
{
	SFXRectangle rectangle;
	SFRMultiDialog::ParamRec param;

    unused(dialog);
    
	switch (result) {
        case SRP16_OK:         // OK button pressed
			SGRSoftkeyToolbox::CloseStandardDialog(dialog);
			
			rectangle.Set(SFXGraphics::GetDeviceRectangle());
			rectangle.Deflate(5, 100);//10

			//
			// display connecting dialog
			//
			param.text = "Connecting...";
			param.ok = "";
			param.cancel = "";
			param.another = "";                      // empty for not dispalying
			param.focus = SFRMultiDialog::FOCUS_OK,  // fucus OK buton
			param.image = SFBImageSmp(null);         // no image
					
			if((_dialog = ::new SFRMultiDialog(rectangle,"Connect", param)) != null)	
			{
				_dialog->RegisterHandler(SFEVT_APP_RESUME, HANDLER_BEFORE, HANDLER_FUNCTION(OnResume));
				_dialog->RegisterHandler(SFEVT_APP_SUSPEND, HANDLER_AFTER, HANDLER_FUNCTION(OnSuspend));
				  
				SGRSoftkeyToolbox::OpenStandardDialog(_dialog, HANDLER_NULL);
			}

			Connect();
            break;
        case SRP16_CANCEL:     // CANCEL button pressed
            SGRSoftkeyToolbox::CloseStandardDialog(dialog);
			_spp(NOTIFY_CONNECT_CANCEL, SFERR_NO_ERROR, _reference);
            break;
    }
}

CALLBACK_IMPLEMENT_SFXHTTPCONNECTION(SGRNetworkDialogSet, ConnectCallback, error)
{
	
	TRACE("CALLBACK_IMPLEMENT_SFXHTTPCONNECTION");
    if (error == SFERR_NO_ERROR) {
        // Get the stream reader
        error = _http.GetStreamReader(1024, &_reader);
        if (error == SFERR_NO_ERROR) {
		// fetch data using callback function
            error = _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
        }
    }
	// notify if error occurred
    if (error != SFERR_NO_ERROR) {
		_http.Close();
		SGRSoftkeyToolbox::CloseStandardDialog(_dialog);
		_spp(NOTIFY_ERROR, error, _reference);
		_errorDialog.Open(HTTP_ERROR_TITLE, HTTP_ERROR_CODE2, DIALOG_OK, this);
    }
	
    return;
}

CALLBACK_IMPLEMENT_SFXBINARYSTREAMREADER(SGRNetworkDialogSet, FetchCallback, error)
{

	TRACE("CALLBACK_IMPLEMENT_SFXBINARYSTREAMREADER");

    SFXBuffer _tempBuffer;  

    if (error == SFERR_NO_ERROR) {
		// get a chunk of data
        _tempBuffer.SetSize(_reader.GetReadableSize());
		_reader.Read(&_tempBuffer);

		// _string holds the returned text in XML format
		error = _receivedBuffer.Add(_tempBuffer);
		
        if (_reader.Ends() && _reader.GetReadableSize() == 0) { // no data remains
           DBGPRINTF("HTTP closed...");
			_http.Close(); // close the connection

			// close the dialog
            SGRSoftkeyToolbox::CloseStandardDialog(_dialog);

			// notify the application
			_connecting = false;
			//DBGPRINTF("Network connect end...");
            _spp(NOTIFY_CONNECT_END, error, _reference);					
        }
        else { // if there is remaining data, fetch again
            _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
        }
    }
    else {
		_http.Close();
		SGRSoftkeyToolbox::CloseStandardDialog(_dialog);
		_spp(NOTIFY_ERROR, error, _reference);
		_errorDialog.Open(HTTP_ERROR_TITLE, HTTP_ERROR_CODE3, DIALOG_OK, this);
    }

    return;
}

HANDLER_IMPLEMENT_VOIDSUSPEND(SGRNetworkDialogSet, OnSuspend, reason, info)
{
    unused(reason);
    unused(info);

	DBGPRINTF("Network Suspend...");

	if (IsConnecting())
	{
		DBGPRINTF("Release resource...");
		_http.Close();
		ClearData();
		//_connecting = false;

		DBGPRINTF("Release resource...done!");
	}
    return;
}

HANDLER_IMPLEMENT_VOIDRESUME(SGRNetworkDialogSet, OnResume, environment)
{
    unused(environment);
	
	if (IsConnecting())
	{	
		DBGPRINTF("Network start again...");
		Connect();
	}
    return;
}

