//
//      SGXListProducts.hpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

/**
***	Project         : Amazon Products Viewing
***	File			: TagTracer.hpp	
***	Description		: Trace the XML file for tag paths
**/

#ifndef __TAGTRACER_HPP
#define __TAGTRACER_HPP

#include <SophiaFramework.hpp>
#include "AWSProduct.hpp"

SFMTYPEDEFCLASS(TagTracer)
SFMTYPEDEFALIAS(SFXStack<TagTracerPtr>, TracerTypeStack)
SFMTYPEDEFCLASS(TagTracerSkip)

typedef Void (AWSProduct::*pt2AWSProductMember)(SFXAnsiString); 

/*
** A tag tracer represents a position in a tag path from the root of an XML document.
** A tag tracer has one parent and zero-to-many children.
** A map of tag tracers (or a tree) contains complete paths from the root of the XML document 
** which represents the process logic (when parsing the XML document, only paths that exists 
** in this map will be processed, otherwise it will be ignored).
** When parsing the XML document, if a path from the root to the current tagname is found, 
** the tag tracer corresponding to this tag name will be place in the top of a stack, if the path 
** is not found, the whole branch from this tagname will be skipped.
*/
class TagTracer {
SFMSEALCOPY(TagTracer)

private:
	SFXHashmap<SFXAnsiString, TagTracerPtr> _traceMap;			// a hash map to keep child tag tracers, the parent tag tracer can trace to the child tag tracers in this map only 
	bool									_haveStartCommand;	// have tag start command?
	bool									_haveEndCommand;	// have tag end command?
    pt2AWSProductMember						_startCommand;		// command to be called at start tag
	pt2AWSProductMember						_endCommand;		// command to be called at end tag

	//callback function
	typedef Void (*CallerProcedureSPP)(SFXAnsiString,  pt2AWSProductMember, VoidPtr);
	CallerProcedureSPP _spp;
public:
	// constructor
    TagTracer(Void);
	TagTracer(pt2AWSProductMember cm);
	TagTracer(pt2AWSProductMember startCm, pt2AWSProductMember endCm);
    // destructor
	virtual		~TagTracer(Void);

	bool HaveStartCommand(Void){return _haveStartCommand;}
	bool HaveEndCommand(Void){return _haveEndCommand;}

	pt2AWSProductMember GetStartCommand(Void){return _startCommand;}
	pt2AWSProductMember GetEndCommand(Void){return _endCommand;}

	// this methos is used to set up a map of tag tracers, 
	// when this method is called, the caller tagtracer is registered to trace to the tracer with this tagname
	Void Trace(SFXAnsiStringConstRef tagname, TagTracerPtr tracer);
	// this method is called when the start of an element is found
	// if the tagname is found in the hashmap it means this tagname will be traced to
	// the found tagtracer for this tagname will be pushed to the stack
	// otherwise, the entire path wil be skipped 
	Void OnStart(SFXAnsiStringConstRef tagname, TracerTypeStackPtr stack, CallerProcedureSPP caller_procedure, VoidPtr caller);
	// this method is called when the end of an element is found
	// it get the top tag tracer (pop), and do the associated command
	Void OnEnd (TracerTypeStackPtr stack, SFXAnsiString data, CallerProcedureSPP caller_procedure, VoidPtr caller);
};

/*
** TagTracerSkip class inherits from TagTracer to present a skipping position in a tag path.
*/
class TagTracerSkip : public TagTracer{
SFMSEALCOPY(TagTracerSkip)

public:
    TagTracerSkip(Void);
    virtual		~TagTracerSkip(Void);
	
	Void OnStart(SFXAnsiStringConstRef tagname, TracerTypeStackPtr stack);
	Void OnEnd (TracerTypeStackPtr stack);
};

#endif // __TAGTRACER_HPP //
